@@ -15,7 +15,9 @@ from matplotlib.gridspec import GridSpec
 from figaro import plot_settings
 from figaro.marginal import marginalise
 from figaro.credible_regions import ConfidenceArea
+from figaro.cumulative import fast_cumulative
 from figaro.utils import recursive_grid
+from figaro.mixture import mixture
 
 # Telling python to ignore empty legend warning from matplotlib
 warnings.filterwarnings("ignore", message = "No artists with labels found to put in legend.  Note that artists whose label start with an underscore are ignored when legend() is called with no argument.")
@@ -98,7 +100,7 @@ class PPPlot(axes.Axes):
         
 projection_registry.register(PPPlot)
 
-def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bounds = None, out_folder = '.', name = 'density', n_pts = 1000, label = None, unit = None, hierarchical = False, show = False, save = True, subfolder = False, true_value = None, true_value_label = '\mathrm{True\ value}', injected_label = '\mathrm{Simulated}', median_label = None):
+def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bounds = None, out_folder = '.', name = 'density', n_pts = 1000, label = None, unit = None, hierarchical = False, show = False, save = True, subfolder = False, true_value = None, true_value_label = '\mathrm{True\ value}', injected_label = '\mathrm{Simulated}', median_label = None, fig = None, colors = ['steelblue', 'darkturquoise', 'mediumturquoise']):
     """
     Plot the recovered 1D distribution along with the injected distribution and samples from the true distribution (both if available).
     
@@ -121,6 +123,12 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
         str true_value_label:            label to assign to the true value marker
         str injected_label:              label to assign to the injected distribution
         str median_label:                label to assign to the reconstruction
+        matplotlib.figure.Figure fig:    figure to use for plotting.
+        list-of-str colors:              list of colors for median, 68% and 90% credible regions
+        
+    
+    Returns:
+        matplotlib.figure.Figure: figure with the plot
     """
     if median_label is None:
         if hierarchical:
@@ -128,6 +136,8 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
         else:
             median_label = '\mathrm{DPGMM}'
     
+    color_med, color_68, color_90 = colors
+    
     all_bounds = np.atleast_2d([d.bounds[0] for d in draws])
     x_min = np.max(all_bounds[:,0])
     x_max = np.min(all_bounds[:,1])
@@ -144,7 +154,10 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
         else:
             x_max = bounds[1]
 
-    fig, ax = plt.subplots()
+    if fig is None:
+        fig, ax = plt.subplots()
+    else:
+        ax = fig.axes[0]
 
     # If samples are available, use them as bounds
     if samples is not None:
@@ -156,9 +169,7 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
     xlim = (x_min, x_max)
     x    = np.linspace(x_min, x_max, n_pts)
     dx   = x[1]-x[0]
-    
     probs = np.array([d.pdf(x) for d in draws])
-    
     percentiles = [50, 5, 16, 84, 95]
     p = {}
     for perc in percentiles:
@@ -174,8 +185,8 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
         ax.set_yscale('log')
     
     # CR
-    ax.fill_between(x, p[95], p[5], color = 'mediumturquoise', alpha = 0.25)
-    ax.fill_between(x, p[84], p[16], color = 'darkturquoise', alpha = 0.25)
+    ax.fill_between(x, p[95], p[5], color = color_90, alpha = 0.25)
+    ax.fill_between(x, p[84], p[16], color = color_68, alpha = 0.25)
     # Selection function (if available)
     if selfunc is not None:
         if callable(selfunc):
@@ -190,17 +201,18 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
             p_x = injected
         if injected_label is not None:
             injected_label = '$'+injected_label+'$'
-        ax.plot(x, p_x, lw = 0.5, color = 'red', label = injected_label)
         if selfunc is not None:
             filtered_p_x = p_x*f_x
-            ax.plot(x, filtered_p_x/np.sum(filtered_p_x*dx), lw = 0.5, color = 'k', label = '$\mathrm{Selection\ effects}$')
+            ax.plot(x, filtered_p_x/np.sum(filtered_p_x*dx), lw = 0.5, color = 'red', label = '$'+injected_label+'\ \\mathrm{(observed)}$')
+        else:
+            ax.plot(x, p_x, lw = 0.5, color = 'red', label = injected_label)
         
     # Median
     if true_value is not None:
         if true_value_label is not None:
             true_value_label = '$'+true_value_label+'$'
-        ax.axvline(true_value, ls = '--', color = 'r', lw = 0.5, label = true_value_label)
-    ax.plot(x, p[50], lw = 0.7, color = 'steelblue', label = '${0}$'.format(median_label))
+        ax.axvline(true_value, ls = '--', color = 'firebrick', dashes = (5,5), lw = 0.5, label = true_value_label)
+    ax.plot(x, p[50], lw = 0.7, color = color_med, label = '${0}$'.format(median_label))
     if label is None:
         label = 'x'
     if unit is None or unit == '':
@@ -240,20 +252,22 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
             plot_folder = out_folder
             log_folder  = out_folder
             txt_folder  = out_folder
-        fig.savefig(Path(log_folder, 'log_{0}.pdf'.format(name)), bbox_inches = 'tight')
-        ax.set_yscale('linear')
-        ax.autoscale(True)
-        if samples is not None:
-            ax.set_xlim(xlim)
-        fig.savefig(Path(plot_folder, '{0}.pdf'.format(name)), bbox_inches = 'tight')
-        np.savetxt(Path(txt_folder, 'prob_{0}.txt'.format(name)), np.array([x, p[50], p[5], p[16], p[84], p[95]]).T, header = 'x 50 5 16 84 95')
+        if not (selfunc is not None and hierarchical):
+            fig.savefig(Path(log_folder, 'log_observed_{0}.pdf'.format(name)), bbox_inches = 'tight')
+            ax.set_yscale('linear')
+            ax.autoscale(True)
+            if samples is not None:
+                ax.set_xlim(xlim)
+            fig.savefig(Path(plot_folder, 'observed_{0}.pdf'.format(name)), bbox_inches = 'tight')
+            np.savetxt(Path(txt_folder, 'prob_observed_{0}.txt'.format(name)), np.array([x, p[50], p[5], p[16], p[84], p[95]]).T, header = 'x 50 5 16 84 95')
     if show:
         ax.set_yscale('linear')
         ax.autoscale(True)
         if samples is not None:
             ax.set_xlim(xlim)
         plt.show()
-    plt.close()
+    if (save or show):
+        plt.close()
     
     # If selection function is available, plot reweighted distribution
     if selfunc is not None:
@@ -284,40 +298,44 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
         ax.set_ylim(bottom = 1e-5, top = np.max(p[95])*1.1)
         ax.legend(loc = 0)
         if save:
-            fig.savefig(Path(log_folder, 'log_true_{0}.pdf'.format(name)), bbox_inches = 'tight')
+            fig.savefig(Path(log_folder, 'log_intrinsic_{0}.pdf'.format(name)), bbox_inches = 'tight')
             ax.set_yscale('linear')
             ax.autoscale(True)
-            fig.savefig(Path(plot_folder, 'true_{0}.pdf'.format(name)), bbox_inches = 'tight')
-            np.savetxt(Path(txt_folder, 'prob_true_{0}.txt'.format(name)), np.array([x, p[50], p[5], p[16], p[84], p[95]]).T, header = 'x 50 5 16 84 95')
+            fig.savefig(Path(plot_folder, 'intrinsic_{0}.pdf'.format(name)), bbox_inches = 'tight')
+            np.savetxt(Path(txt_folder, 'prob_intrinsic_{0}.txt'.format(name)), np.array([x, p[50], p[5], p[16], p[84], p[95]]).T, header = 'x 50 5 16 84 95')
         if show:
             ax.set_yscale('linear')
             ax.autoscale(True)
             plt.show()
-        plt.close()
     
+    if (save or show):
+        plt.close()
+    return fig
 
-def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name = 'density', labels = None, units = None, hierarchical = False, show = False, save = True, subfolder = False, n_pts = 200, true_value = None, levels = [0.5, 0.68, 0.9], scatter_points = False, median_label = None):
+def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name = 'density', labels = None, units = None, hierarchical = False, show = False, save = True, subfolder = False, n_pts = 200, true_value = None, levels = [0.5, 0.68, 0.9], scatter_points = False, median_label = None, fig = None, colors = ['steelblue', 'darkturquoise', 'mediumturquoise'], colormap = 'Blues'):
     """
     Plot the recovered multidimensional distribution along with samples from the true distribution (if available) as corner plot.
     
     Arguments:
-        iterable draws:         container for mixture instances
-        int dim:                number of dimensions
-        np.ndarray samples:     samples from the true distribution (if available)
-        iterable bounds:        bounds for the recovered distribution. If None, bounds from mixture instances are used.
-        str or Path out_folder: output folder
-        str name:               name to be given to outputs
-        list-of-str labels:     LaTeX-style quantity label, for plotting purposes
-        list-of-str units:      LaTeX-style quantity unit, for plotting purposes
-        bool hierarchical:      hierarchical inference, for plotting purposes
-        bool save:              whether to save the plot or not
-        bool show:              whether to show the plot during the run or not
-        bool subfolder:         whether to save in a dedicated subfolder
-        int n_pts:              number of grid points (same for each dimension)
-        iterable true_value:    true value to plot
-        iterable levels:        credible levels to plot
-        bool scatter_points:    scatter samples on 2d plots
-        str median_label:       label to assign to the reconstruction
+        iterable draws:                container for mixture instances
+        np.ndarray samples:            samples from the true distribution (if available)
+        iterable bounds:               bounds for the recovered distribution. If None, bounds from mixture instances are used.
+        str or Path out_folder:        output folder
+        str name:                      name to be given to outputs
+        list-of-str labels:            LaTeX-style quantity label, for plotting purposes
+        list-of-str units:             LaTeX-style quantity unit, for plotting purposes
+        bool hierarchical:             hierarchical inference, for plotting purposes
+        bool save:                     whether to save the plot or not
+        bool show:                     whether to show the plot during the run or not
+        bool subfolder:                whether to save in a dedicated subfolder
+        int n_pts:                     number of grid points (same for each dimension)
+        iterable true_value:           true value to plot
+        iterable levels:               credible levels to plot
+        bool scatter_points:           scatter samples on 2d plots
+        str median_label:              label to assign to the reconstruction
+        matplotlib.figure.Figure fig:  figure to use for plotting. Must have (dim,dim) axes.
+        list-of-str colors:            list of colors for median, 68% and 90% credible regions
+        str colormap:                  colormap for contour plots
     
     Returns:
         matplotlib.figure.Figure: figure with the plot
@@ -329,6 +347,9 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
             median_label = '\mathrm{(H)DPGMM}'
         else:
             median_label = '\mathrm{DPGMM}'
+    
+    color_med, color_68, color_90 = colors
+
     if labels is None:
         labels = ['$x_{'+'{0}'.format(i+1)+'}$' for i in range(dim)]
     else:
@@ -371,7 +392,10 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
     plotdim = factor * dim + factor * (K - 1.0) * whspace
     dim_plt = lbdim + plotdim + trdim
     
-    fig, axs = plt.subplots(K, K, figsize=(dim_plt, dim_plt))
+    if fig is None:
+        fig, axs = plt.subplots(K, K, figsize=(dim_plt, dim_plt))
+    else:
+        axs = np.array(fig.axes).reshape(dim,dim)
     # Format the figure.
     lb = lbdim / dim_plt
     tr = (lbdim + plotdim) / dim_plt
@@ -387,7 +411,13 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
         # Marginalise over all uninterested columns
         dims = list(np.arange(dim))
         dims.remove(column)
-        marg_draws = marginalise(draws, dims)
+        try:
+            if np.all([isinstance(d, mixture) for d in draws]):
+                marg_draws = marginalise(draws, dims)
+            else:
+                marg_draws = np.array([d.marginalise(dims) for d in draws])
+        except AttributeError:
+            raise AttributeError("The provided instances do not have a marginalise() method")
         # Credible regions
         lim = bounds[column]
         if samples is not None and not ext_bounds:
@@ -409,12 +439,12 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
             p[perc] = p[perc]/norm
             
         # CR
-        ax.fill_between(x, p[95], p[5], color = 'mediumturquoise', alpha = 0.25)
-        ax.fill_between(x, p[84], p[16], color = 'darkturquoise', alpha = 0.25)
+        ax.fill_between(x, p[95], p[5], color = color_90, alpha = 0.25)
+        ax.fill_between(x, p[84], p[16], color = color_68, alpha = 0.25)
         if true_value is not None:
             if true_value[column] is not None:
-                ax.axvline(true_value[column], c = 'orangered', lw = 0.5)
-        ax.plot(x, p[50], lw = 0.7, color = 'steelblue', label = '${0}$'.format(median_label))
+                ax.axvline(true_value[column], c = 'firebrick', lw = 0.5)
+        ax.plot(x, p[50], lw = 0.7, color = color_med, label = '${0}$'.format(median_label))
         ax.set_yticks([])
         if column == K - 1:
             if labels is not None:
@@ -443,7 +473,13 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
             dims = list(np.arange(dim))
             dims.remove(column)
             dims.remove(row)
-            marg_draws = marginalise(draws, dims)
+            try:
+                if np.all([isinstance(d, mixture) for d in draws]):
+                    marg_draws = marginalise(draws, dims)
+                else:
+                    marg_draws = np.array([d.marginalise(dims) for d in draws])
+            except:
+                raise AttributeError("The provided instances do not have a marginalise() method")
             
             # Credible regions
             lim = bounds[[row, column]]
@@ -467,22 +503,22 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
             with np.errstate(divide = 'ignore'):
                 logmedian = np.nan_to_num(np.log(median), nan = -np.inf, neginf = -np.inf)
             _,_,levs = ConfidenceArea(logmedian, x, y, adLevels=levels)
-            ax.contourf(Y, X, np.exp(logmedian), cmap = 'Blues', levels = 100)
+            ax.contourf(Y, X, np.exp(logmedian), cmap = colormap, levels = 100)
             if true_value is not None:
                 if true_value[row] is not None:
-                    ax.axhline(true_value[row], c = 'orangered', lw = 0.5)
+                    ax.axhline(true_value[row], c = 'firebrick', lw = 0.5)
                 if true_value[column] is not None:
-                    ax.axvline(true_value[column], c = 'orangered', lw = 0.5)
+                    ax.axvline(true_value[column], c = 'firebrick', lw = 0.5)
                 if true_value[column] is not None and true_value[row] is not None:
-                    ax.plot(true_value[column], true_value[row], color = 'orangered', marker = 's', ms = 3)
-            c1 = ax.contour(Y, X, logmedian, np.sort(levs), colors='steelblue', linewidths=0.3)
+                    ax.plot(true_value[column], true_value[row], color = 'firebrick', marker = 's', ms = 3)
+            c1 = ax.contour(Y, X, logmedian, np.sort(levs), colors=color_med, linewidths=0.3)
             if plot_settings.tex_flag:
                 ax.clabel(c1, fmt = {l:'{0:.0f}\\%'.format(100*s) for l,s in zip(c1.levels, np.sort(levels)[::-1])}, fontsize = 3)
             else:
                 ax.clabel(c1, fmt = {l:'{0:.0f}\%'.format(100*s) for l,s in zip(c1.levels, np.sort(levels)[::-1])}, fontsize = 3)
             # Samples (if available)
             if samples is not None and scatter_points:
-                ax.scatter(samples[:,column], samples[:,row], marker = '+', c = 'orangered', linewidths = 1)
+                ax.scatter(samples[:,column], samples[:,row], marker = '+', c = 'firebrick', linewidths = 1)
             # Ticks
             xticks = np.linspace(lim[1,0], lim[1,1], 5)
             ax.set_xticks(xticks)
@@ -515,10 +551,11 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
                 except FileExistsError:
                     pass
             fig.savefig(Path(out_folder, 'density', '{0}.pdf'.format(name)), bbox_inches = 'tight')
-    plt.close()
+    if (save or show):
+        plt.close()
     return fig
     
-def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', name = 'density', label = None, unit = None, show = False, save = True, subfolder = False, true_value = None, true_value_label = '\mathrm{True\ value}', injected_label = '\mathrm{Simulated}', median_label = '\mathrm{Median}', logx = False, logy = False):
+def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', name = 'density', label = None, unit = None, show = False, save = True, subfolder = False, true_value = None, true_value_label = '\mathrm{True\ value}', injected_label = '\mathrm{Simulated}', median_label = '\mathrm{Median}', logx = False, logy = False, colors = ['steelblue','darkturquoise','mediumturquoise'], fig = None):
     """
     Plot a 1D distribution along with samples from the true distribution (if available).
     Differently from plot_median_cr, this method requires the distribution to be already evaluated.
@@ -542,6 +579,8 @@ def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', na
         str median_label:                label to assign to the median distribution
         bool logx:                       x log scale
         bool logy:                       y log scale
+        list-of-str colors:              list of colors for median, 68% and 90% credible regions
+        matplotlib.figure.Figure fig:    figure to use for plotting.
     """
     
     if not np.shape(x)[0] == np.shape(draws)[-1]:
@@ -552,7 +591,12 @@ def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', na
     for perc in percentiles:
         p[perc] = np.percentile(draws, perc, axis = 0)
     
-    fig, ax = plt.subplots()
+    color_med, color_68, color_90 = colors
+    
+    if fig is None:
+        fig, ax = plt.subplots()
+    else:
+        ax = fig.axes[0]
     
     # Samples (if available)
     if samples is not None:
@@ -561,8 +605,8 @@ def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', na
         ylim = ax.get_ylim()
     
     # CR
-    ax.fill_between(x, p[95], p[5], color = 'mediumturquoise', alpha = 0.25)
-    ax.fill_between(x, p[84], p[16], color = 'darkturquoise', alpha = 0.25)
+    ax.fill_between(x, p[95], p[5], color = color_90, alpha = 0.25)
+    ax.fill_between(x, p[84], p[16], color = color_68, alpha = 0.25)
     # Injection (if available)
     if injected is not None:
         if callable(injected):
@@ -577,10 +621,10 @@ def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', na
     if true_value is not None:
         if true_value_label is not None:
             true_value_label = '$'+true_value_label+'$'
-        ax.axvline(true_value, ls = '--', color = 'r', lw = 0.5, label = true_value_label)
+        ax.axvline(true_value, ls = '--', color = 'firebrick', lw = 0.5, dashes = (5,5), label = true_value_label)
     if median_label is not None:
         median_label = '$'+median_label+'$'
-    ax.plot(x, p[50], lw = 0.7, color = 'steelblue', label = median_label)
+    ax.plot(x, p[50], lw = 0.7, color = color_med, label = median_label)
     if label is None:
         label = 'x'
     if unit is None or unit == '':
@@ -627,8 +671,9 @@ def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', na
         if samples is not None:
             ax.set_xlim(xlim)
         plt.show()
-    plt.close()
-    
+    if (save or show):
+        plt.close()
+    return fig
 
 def plot_n_clusters_alpha(n_cl, alpha, out_folder = '.', name = 'event', show = False, save = True):
     """
@@ -645,10 +690,10 @@ def plot_n_clusters_alpha(n_cl, alpha, out_folder = '.', name = 'event', show =
     fig, ax = plt.subplots()
     ax1 = ax.twinx()
     ax.plot(np.arange(1, len(n_cl)+1), n_cl, ls = '--', marker = '', lw = 0.7, color = 'k')
-    ax1.plot(np.arange(1, len(alpha)+1), alpha, ls = '--', marker = '', lw = 0.7, color = 'r')
+    ax1.plot(np.arange(1, len(alpha)+1), alpha, ls = '--', marker = '', lw = 0.7, color = 'firebrick')
     ax.set_xlabel('$t$')
     ax.set_ylabel('$N_{\mathrm{cl}}(t)$', color = 'k')
-    ax1.set_ylabel('$\alpha(t)$', color = 'r')
+    ax1.set_ylabel('$\alpha(t)$', color = 'firebrick')
     if show:
         plt.show()
     if save:
@@ -694,28 +739,36 @@ def pp_plot_cdf(draws, injection, n_points = 1000, out_folder = '.', name = 'eve
         fig.savefig(Path(out_folder, '{0}_ppplot.pdf'.format(name)), bbox_inches = 'tight')
     plt.close()
 
-def pp_plot_levels(CR_levels, median_CR = None, out_folder = '.', name = 'MDC', show = False, save = True):
+def pp_plot_levels(CR_levels, median_CR = None, out_folder = '.', name = 'MDC', show = False, save = True, fig = None, color = 'steelblue', label = None):
     """
     Make pp-plot.
     
     Arguments:
-        iterable CR:            2D array with credible levels for each event
-        iterable median_CR:     credible levels of medians
-        str or Path out_folder: output folder
-        str name:               name to be given to outputs
-        bool save:              whether to save the plot or not
-        bool show:              whether to show the plot during the run or not
+        iterable CR:                  2D array with credible levels for each event
+        iterable median_CR:           credible levels of medians
+        str or Path out_folder:       output folder
+        str name:                     name to be given to outputs
+        bool save:                    whether to save the plot or not
+        bool show:                    whether to show the plot during the run or not
+        matplotlib.figure.Figure fig: figure to use for plotting.
+        str color:                    color for the main line
     """
     CR_levels = np.atleast_1d(CR_levels)
     if len(CR_levels.shape) > 1:
         CR_levels = CR_levels.T
     n_evs     = CR_levels.shape[-1]
     L         = np.linspace(0,1,n_evs+2)
-    
-    fig = plt.figure()
-    ax  = fig.add_subplot(111, projection = 'pp_plot')
-    ax.add_confidence_band(n_evs, zorder = n_evs)
-    ax.add_diagonal(zorder = n_evs+1)
+    if label is not None:
+        label = '$'+label+'$'
+    else:
+        label = r'$\\mathrm{Median}$'
+    if fig is None:
+        fig = plt.figure()
+        ax  = fig.add_subplot(111, projection = 'pp_plot')
+        ax.add_confidence_band(n_evs, zorder = n_evs)
+        ax.add_diagonal(zorder = n_evs+1)
+    else:
+        ax = fig.axes[0]
     if len(CR_levels.shape) > 1:
         sorted = []
         for cr in CR_levels:
@@ -724,30 +777,32 @@ def pp_plot_levels(CR_levels, median_CR = None, out_folder = '.', name = 'MDC',
                 c  = 'lightsteelblue'
             else:
                 lw = 0.6
-                c  = 'steelblue'
+                c  = color
             x = np.append(0, np.append(cr, 1))
             ax.plot(np.sort(x), L, lw = lw, alpha = 0.5, color = c)
         if median_CR is not None:
             x = np.append(0, np.append(median_CR, 1))
-            ax.plot(np.sort(x), L, lw = 0.8, color = 'steelblue', label = '$\mathrm{Median}$', zorder = n_evs+2)
+            ax.plot(np.sort(x), L, lw = 0.8, color = color, label = label, zorder = n_evs+2)
         # Add label for draws
         handles, labels = ax.get_legend_handles_labels()
-        line = Line2D([0], [0], label='$\mathrm{Draws}$', lw = lw, color = c)
+        line = Line2D([0], [0], label='$\\mathrm{Draws}$', lw = lw, color = c)
         handles.extend([line])
         ax.legend(handles=handles, loc = 0, frameon = False)
     else:
         x = np.append(0, np.append(CR_levels, 1))
-        ax.plot(np.sort(x), L, lw = 0.8, color = 'steelblue', zorder = n_evs+2)
+        ax.plot(np.sort(x), L, lw = 0.8, color = color, zorder = n_evs+2, label = label)
     # Maquillage
     ax.set_xlabel('$P$')
-    ax.set_ylabel('$\mathrm{Fraction\ of\ events\ within\ }CR_P$')
+    ax.set_ylabel(r'$\mathrm{Fraction\ of\ events\ within\ }CR_P$')
     if show:
         plt.show()
     if save:
         fig.savefig(Path(out_folder, '{0}_ppplot.pdf'.format(name)), bbox_inches = 'tight')
-    plt.close()
+    if (save or show):
+        plt.close()
+    return fig
 
-def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, solid = False, overlap = 1., xlabel = None, ylabel = None, xunit = None, yunit = None, colormap = 'coolwarm', out_folder = '.', name = 'joyplot', subfolder = False, show = False, save = True, joy = False):
+def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, solid = False, overlap = 1., xlabel = None, ylabel = None, xunit = None, yunit = None, colormap = 'coolwarm', out_folder = '.', name = 'joyplot', subfolder = False, show = False, save = True, joy = False, colorbar = True, histogram = False):
     """
     Make a joyplot (also known as ridgeline plot) of a set of distributions.
     Heavily inspired by leotac's JoyPy (https://github.com/leotac/joypy).
@@ -804,7 +859,7 @@ def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, so
     gs = GridSpec(len(y_values), 17, figure = fig)
     _axes = []
     [_axes.append(fig.add_subplot(gs[i, :-1])) for i in range(len(y_values))]
-    if not joy:
+    if not joy and colorbar:
         _axes.append(fig.add_subplot(gs[:, -1]))
         # Global colorbar
         cbar    = fig.colorbar(cmappable, cax = _axes[-1])
@@ -817,27 +872,36 @@ def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, so
             c = next(color)
         else:
             c = 'w'
-        percentiles = [50, 16, 84]
-        p = {}
-        if len(np.shape(draws)) == 3:
-            for perc in percentiles:
-                p[perc] = np.percentile(dd, perc, axis = 0)
-            if credible_regions and not joy:
-                ax.fill_between(x_values, p[95], p[5], color = c, alpha = 0.25)
-                ax.fill_between(x_values, p[84], p[16], color = c, alpha = 0.25)
+        if not histogram:
+            percentiles = [50, 16, 84]
+            p = {}
+            if len(np.shape(draws)) == 3:
+                for perc in percentiles:
+                    p[perc] = np.percentile(dd, perc, axis = 0)
+                if credible_regions and not joy:
+                    ax.fill_between(x_values, p[84], p[16], color = c, alpha = 0.25)
+            else:
+                p[50] = dd
+            if fill and not credible_regions:
+                if solid:
+                    alpha = 1
+                else:
+                    alpha = 0.25
+                if not joy:
+                    ax.fill_between(x_values, p[50], np.zeros(len(x_values)), color = c, alpha = alpha)
+                else:
+                    ax.fill_between(x_values, p[50], np.zeros(len(x_values)), color = 'k', alpha = 1)
+            ax.plot(x_values, p[50], clip_on = True, c = c)
         else:
-            p[50] = dd
-        if fill and not credible_regions:
-            if solid:
-                alpha = 1
-            else:
-                alpha = 0.25
-            if not joy:
-                ax.fill_between(x_values, p[50], np.zeros(len(x_values)), color = c, alpha = alpha)
-            else:
-                ax.fill_between(x_values, p[50], np.zeros(len(x_values)), color = 'k', alpha = 1)
-        ax.plot(x_values, p[50], clip_on = True, c = c)
-
+            if fill:
+                histtype = 'stepfilled'
+                if solid:
+                    alpha = 1
+                else:
+                    alpha = 0.25
+                ax.hist(dd, histtype = histtype, alpha = alpha, density = True, color = c)
+            ax.hist(dd, histtype = 'step', alpha = 1, density = True, color = c)
+            ax.plot(x_values, np.zeros(len(x_values)), alpha = 1, color = c, marker = '', lw = 1.)
         # Setup the current axis: transparency, labels, spines.
         ax.yaxis.grid(False)
         ax.patch.set_alpha(0)
