@@ -1,9 +1,18 @@
 from matplotlib import rcParams
-from matplotlib.pyplot import hist
-from distutils.spawn import find_executable
+from matplotlib import pyplot as plt
+from shutil import which
+import inspect
+
+def get_default_args(func):
+    signature = inspect.signature(func)
+    return {
+        k: v.default
+        for k, v in signature.parameters.items()
+        if v.default is not inspect.Parameter.empty
+    }
 
 tex_flag = False
-if find_executable('latex'):
+if which('latex') is not None:
     tex_flag = True
     rcParams["text.usetex"] = True
 rcParams["xtick.labelsize"] = 14
@@ -22,8 +31,13 @@ rcParams["hist.bins"]       = "sqrt"
 rcParams["savefig.bbox"]    = "tight"
 rcParams["contour.negative_linestyle"] = "solid"
 
-# Better way of doing this?
-histdefaults = list(hist.__defaults__)
-histdefaults[2] = True   # density
-histdefaults[6] = 'step' # histtype
-hist.__defaults__ = tuple(histdefaults)
+def nicer_hist(func):
+    def decorated_func(*args, **kwargs):
+        if not 'density' in kwargs.keys():
+            kwargs['density'] = True
+        if not 'histtype' in kwargs.keys():
+            kwargs['histtype'] = 'step'
+        return func(*args, **kwargs)
+    return decorated_func
+
+plt.hist = nicer_hist(plt.hist)
