@@ -9,7 +9,7 @@ See notes there for cumulative of the unit normal distribution.
 """
 
 def transform_to_probit(x, bounds):
-    '''
+    """
     Coordinate change into probit space.
     cdf_normal is the cumulative distribution function of the unit normal distribution.
     WARNING: returns NAN if x is not in [xmin, xmax].
@@ -23,7 +23,7 @@ def transform_to_probit(x, bounds):
         
     Returns:
         np.ndarray: sample(s)
-    '''
+    """
     dbounds = bounds[:,1]-bounds[:,0]
     sigma   = dbounds*0.34
     cdf = (x - bounds[:,0])/dbounds
@@ -31,7 +31,7 @@ def transform_to_probit(x, bounds):
     return o
 
 def transform_from_probit(x, bounds):
-    '''
+    """
     Coordinate change from probit to natural space.
     cdf_normal is the cumulative distribution function of the unit normal distribution.
     
@@ -43,20 +43,44 @@ def transform_from_probit(x, bounds):
         
     Returns:
         np.ndarray: sample(s)
-    '''
+    """
     dbounds = bounds[:,1]-bounds[:,0]
     sigma   = dbounds*0.34
     cdf = 0.5*(1.0+erf(x/(np.sqrt(2.0)*sigma)))
     o = bounds[:,0]+dbounds*cdf
     return o
 
-def probit_log_jacobian(x, bounds):
+def probit_log_jacobian(x, bounds, flag = True):
+    """
+    Jacobian of the probit transformation
+    
+    Arguments:
+        np.ndarray x:      sample(s) to evaluate the jacobian at
+        np.ndarray bounds: limits for each dimension (2d array, [[xmin, xmax], [ymin, ymax]...])
+        bool flag:         whether to skip the evaluation
+    
+    Returns:
+        np.ndarray: log jacobian (zeros if flag is False)
+    """
+    if not flag:
+        return np.zeros(len(x))
     dbounds = bounds[:,1]-bounds[:,0]
     sigma   = dbounds*0.34
     res     = -0.5*(x/sigma)**2-0.5*(log2PI)+np.log(dbounds)-np.log(sigma)
     return res
 
 def probit_logJ(x, bounds, flag = True):
+    """
+    Jacobian of the probit transformation marginalised over dimensions
+    
+    Arguments:
+        np.ndarray x:      sample(s) to evaluate the jacobian at
+        np.ndarray bounds: limits for each dimension (2d array, [[xmin, xmax], [ymin, ymax]...])
+        bool flag:         whether to skip the evaluation
+    
+    Returns:
+        np.ndarray: log jacobian (zeros if flag is False)
+    """
     if not flag:
         return np.zeros(len(x))
     dbounds = bounds[:,1]-bounds[:,0]
@@ -64,9 +88,20 @@ def probit_logJ(x, bounds, flag = True):
     res     = np.sum(-0.5*(x/sigma)**2-0.5*log2PI+np.log(dbounds)-np.log(sigma), axis = -1)
     return res
 
-def log_gradient_inv_jacobian(x, bounds, flag = True):
+def gradient_inv_jacobian(x, bounds, flag = True):
+    """
+    logarithmic gradient of the probit transformation Jacobian
+    
+    Arguments:
+        np.ndarray x:      sample(s) to evaluate the jacobian at
+        np.ndarray bounds: limits for each dimension (2d array, [[xmin, xmax], [ymin, ymax]...])
+        bool flag:         whether to skip the evaluation
+    
+    Returns:
+        np.ndarray: log jacobian (ones if flag is False)
+    """
     if not flag:
-        return np.zeros(len(x))
+        return np.ones(len(x))
     dbounds = bounds[:,1]-bounds[:,0]
     sigma   = dbounds*0.34
-    return x/sigma
+    return x/sigma**2.
